package com.chinacreator.video.service;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.CommConfDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.DictionaryDao;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.URLUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;

@Service
public class CancelForNengkaiService {

    private static final Logger log = LoggerFactory.getLogger("cancelForNengkai");

    @Autowired
    DictionaryDao dictionaryDao;

    @Autowired
    CpSpDao cpSpDao;

    @Autowired
    CommConfDao commConfDao;

    @Autowired
    NengkaiService nengkaiService;

    public String cancel(JSONObject object) throws Exception{
        String data = "";
        HashMap<String,String> logmap = new HashMap<>();
         try {
             check(object);
            String cancelUrl = dictionaryDao.getValue("commonOrderUrl")+"cancelOrder.do?";
            cancelUrl += this.getParams(object);
            logmap.put("url",cancelUrl);
            String result = URLUtil.get(cancelUrl);
//            String result ="{\"resultcode\":\"0\",\"errorinfo\":\"成功\"}";
            logmap.put("result",result);
            if(!"0".equals(JSON.parseObject(result).get("resultcode"))) {
                throw new BussinessException(JSON.parseObject(result).get("resultcode").toString(),JSON.parseObject(result).get("errorinfo").toString());
            }
            String canceltime = commConfDao.getCurrtime2();
            String endtime = commConfDao.getEndDayOfCurrentMonth2();
            HashMap<String,String> map = new HashMap<>();
            map.put("canceltime",canceltime);
            map.put("endtime",endtime);
            logmap.put("data",JSON.toJSONString(map));
            data = nengkaiService.encrypt(JSON.toJSONString(map),object.getString("cpidChannel"));
            logmap.put("encryptdata",data);
        }catch (Exception e){
            if(e instanceof BussinessException) {
                throw e;
            }else {
                e.printStackTrace();
                String errmsg = e.getMessage();
                if( errmsg != null && errmsg.indexOf("videoif") != -1){
                    throw new BussinessException("80003", "系统忙，请稍候再试。");
                }else{
                    throw new BussinessException("80003", "出现异常，"+e.getMessage());
                }
            }
        }finally {
             log.info(JSON.toJSONString(logmap));
         }
        return data;
    }

    private String getParams(JSONObject object) throws Exception{
        String params = "";
        try {
            String spid = object.getString("productid");
            List<HashMap> spInfo = commConfDao.findSpInfo(spid);
            if (spInfo==null||spInfo.size()==0){
                throw new BussinessException("80013", "产品编码异常");
            }
            String cpid = spInfo.get(0).get("CPID").toString();
            String userid = object.getString("usermob");
            String timestamp = System.currentTimeMillis()/1000+"";
            String seqnumber = "";
            String subchannel = "";
            String subtype = "";
            String channel = object.getString("cpidChannel");
            if(StringUtils.isEmpty(seqnumber)){
                seqnumber = "";
            }
            String seqnumberParams = "";
            if(!"".equals(seqnumber)){
                seqnumberParams = "&seqnumber="+seqnumber;
            }
            String pwd = cpSpDao.getCpSp(cpid, spid).getPwd(); //TB_CP_ACCOUNT_CONFIG表NETPWD字段

            params += "ordertype=0&access="+"1"+"&apptype=2&channel="+channel
                        +"&subchannel="+subchannel
                        +"&cpid="+cpid+"&spid="+spid+"&timestamp="+timestamp+"&token="+""+"&subtype="+subtype;
            userid = DesUtil.encode(userid, pwd); //手机号码加密
            params += "&userid="+ URLEncoder.encode(userid, "utf-8"); //重新编码
            params += seqnumberParams;
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof BussinessException){
                throw e;
            }else {
                throw new BussinessException("80013", "组建调用参数出现异常，"+e.getMessage());
            }
        }
        return params;
    }

    public void check(JSONObject object)throws Exception{
        if (StringUtils.isEmpty(object.getString("productid"))||
        StringUtils.isEmpty(object.getString("usermob"))){
            throw new BussinessException("80013", "参数异常缺失");
        }
    }
}
