package com.chinacreator.video.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.ActivityBean;
import com.chinacreator.video.bean.ProJinVokeUrlBean;
import com.chinacreator.video.bean.ReceiveEquityLog;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.DictionaryDao;
import com.chinacreator.video.mapper.ProJinVokeUrlDao;
import com.chinacreator.video.mapper.QureyEquityDao;
import com.chinacreator.video.restful.dao.VipJoinSpidDao;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.URLUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 查询权益信息
 *
 * @Author shuiying.ou
 * @Date 2023-04-12
 */
@Service
public class QureyEquityService {

    private static final Logger log = LoggerFactory.getLogger("queryOrderForNengkai");

    @Autowired
    QureyEquityDao qureyEquityDao;

    @Autowired
    VipJoinSpidDao vipJoinSpidDao;

    @Autowired
    DictionaryDao dictionaryDao;

    @Autowired
    NengkaiService nengkaiService;

    @Autowired
    ProJinVokeUrlDao proJinVokeUrlDao;

    /**
     * 根据手机号码与spid查询产品下的权益信息
     *
     * @param userid 手机号
     * @param spid   产品编码
     * @return 权益信息集合
     */
    public List<Map<String, String>> getVipInfo(String userid, String spid, String cpid, String orderchannel, String orderchannel2) throws BussinessException {

        ArrayList<Map<String, String>> list = new ArrayList<>();
        HashMap<String, String> map = new HashMap<>();

        try {
            String activetype = qureyEquityDao.getActivetype(spid);
            if (!StringUtils.isEmpty(activetype)) {
                List<ActivityBean> activityList = qureyEquityDao.getActivetypeInfo(userid, cpid, spid, activetype);
                if (activityList.size() > 0) {
                    for (ActivityBean activityBean : activityList) {

                        // 单产品为spid，复合产品为子产品spid
                        String spidSon = activityBean.getFhspid() == null ? activityBean.getSpid() : activityBean.getFhspid();

                        Map<String, String> infoMap = qureyEquityDao.getActiveConfig(spidSon, activetype);
                        if (infoMap != null) {
                            String vipname = infoMap.get("VIPNAME");

                            map.put("equitycode", activityBean.getFhspid() == null ? activityBean.getSpid() : activityBean.getFhspid());   // 权益编码
                            map.put("equityname", vipname);   // 权益名称

                            String systime = new SimpleDateFormat("yyyy-MM").format(new Date());

                            // 权益领取状态   0已领取   1未领取   2 领取中
                            if ("0".equals(activityBean.getVipstatus())) {
                                if (!StringUtils.isEmpty(activityBean.getViptime()) && systime.equals(activityBean.getViptime().substring(0, 7))) {
                                    log.info("userid=>" + userid + "，spid=>" + spid + "，当月已领取");
                                    map.put("equitystatus", "0");
                                } else {
                                    map.put("equitystatus", "1");
                                }
                            } else if ("990099".equals(activityBean.getVipstatus()) || "3".equals(activityBean.getVipstatus())) {
                                map.put("equitystatus", "2");
                            } else {
                                map.put("equitystatus", "1");
                            }

                            // 判断次月是否允许手动领取
                            // 本月是否能领取   0不能领取     1能领取
                            if (vailVipJoinSpidConf(userid, spid, orderchannel, orderchannel2, activityBean)) {
                                map.put("isreceive", "1");
                            } else {
                                map.put("isreceive", "0");
                            }

                            // 领取时间，已领取为必填yyyy-MM-dd HH:mm:ss
                            if ("0".equals(map.get("equitystatus"))) {
                                map.put("equitytime", activityBean.getViptime());
                            }

                            list.add(map);
                        }else {
                            log.info("spid :" + spid + ",cpid :" + cpid + "activetype :" + activetype + "TB_ACTIVITY_VIP_CONFIG缺失配置");
                        }
                    }
                } else {
                    log.info("spid :" + spid + ",cpid :" + cpid + "，activetype :" + activetype + "无活动关系");
                }
            }
            return list;
        } catch (BussinessException e) {
            e.printStackTrace();
            log.info("查询权益信息异常" + e.getMessage() + "userid" + userid + "spid" + spid);
            throw e;
        }
    }

    private Boolean vailVipJoinSpidConf(String userid, String spid, String orderchannel, String orderchannel2, ActivityBean activityBean) throws BussinessException {
        boolean contains = false;
        String ordertime = activityBean.getOrdertime();
        try {
            //获取手动赠送配置表的信息
            List<HashMap> dataList = vipJoinSpidDao.getVipJoinSpidConf(spid);
            if (dataList != null && dataList.size() > 0) {

                log.info("手动领取会员配置：" + dataList + "，orderchannel2=>" + orderchannel2 + "，orderchannel=>" + orderchannel + "，userid=>" + userid + "，spid=>" + spid);
                //子渠道标识或主渠道标识不为空
                if (!StringUtils.isEmpty(orderchannel2) || !StringUtils.isEmpty(orderchannel)) {
                    //渠道是否有配置
                    for (HashMap hm : dataList) {
                        if ("0".equals(hm.get("HASSUBCHANNEL"))) {
                            //子渠道
                            //配置渠道与子渠道相同，SPID相同，当前月份与已领取月份不相同
                            if (!StringUtils.isEmpty(orderchannel2)
                                    && (orderchannel2.equals(hm.get("CHANNEL")))
                                    && spid.equals(hm.get("SPID"))
                                    && this.valiConfDate(userid, spid, activityBean, hm)) {
                                contains = true;
                                break;
                            }
                        } else {
                            //主渠道
                            if (!StringUtils.isEmpty(orderchannel)
                                    && ("*".equals(hm.get("CHANNEL")))
                                    && spid.equals(hm.get("SPID"))
                                    && this.valiConfDate(userid, spid, activityBean, hm)) {//主渠道
                                //配置渠道为 * ，SPID相同，当前月份与已领取月份不相同
                                if (!StringUtils.isEmpty(orderchannel)
                                        && ("*".equals(hm.get("CHANNEL")))
                                        && spid.equals(hm.get("SPID"))
                                        && this.valiConfDate(userid, spid, activityBean, hm)) {
                                    String time = "20221101000000";
                                    final long time_long = Long.parseLong(time);
                                    final long ordertime_long = Long.parseLong(ordertime);
                                    /*
                                     * 修改新逻辑
                                     * 11.1之前订购的不变
                                     * 1011,1012,1016,1047,1048,1049,1051,1052（根据业务变化）
                                     * 11.1之后订购的，除部分产品的几个特殊渠道，以上spid由续送改为手动领取
                                     * */
                                    if (!StringUtils.isEmpty(ordertime)) {
                                        String contractspids = dictionaryDao.getValue("contractspids");
                                        if (!StringUtils.isEmpty(contractspids) && contractspids.contains(spid)) {
                                            if (ordertime_long >= time_long) {
                                                String specialchannels = dictionaryDao.getValue("specialchannels_" + spid);
                                                if (!StringUtils.isEmpty(specialchannels)) {
                                                    if (specialchannels.contains(orderchannel2)) {
                                                        log.info("该产品渠道为需续送的特殊渠道" + spid + "=======>" + orderchannel2);
                                                    } else {
                                                        contains = true;
                                                    }
                                                } else {
                                                    contains = true;
                                                }
                                            } else {
                                                // 11.1之前领取配置无*的概念
                                                continue;
                                            }
                                        } else {
                                            contains = true;
                                        }
                                    } else {
                                        log.info("该数据无订购关系" + userid + "=====>" + spid);
                                    }
                                    break;
                                } else if (!StringUtils.isEmpty(orderchannel)
                                        && (orderchannel.equals(hm.get("CHANNEL")))
                                        && spid.equals(hm.get("SPID"))
                                        && this.valiConfDate(userid, spid, activityBean, hm)) {
                                    //配置渠道与主渠道相同，SPID相同，当前月份与已领取月份不相同
                                    contains = true;
                                    break;
                                }
                            } else if (!StringUtils.isEmpty(orderchannel)
                                    && (orderchannel.equals(hm.get("CHANNEL")))
                                    && spid.equals(hm.get("SPID"))
                                    && this.valiConfDate(userid, spid, activityBean, hm)) {
                                //配置渠道与主渠道相同，SPID相同，当前月份与已领取月份不相同
                                contains = true;
                                break;
                            }
                        }
                    }
                } else {//子渠道标识他和主渠道标识都为空
                    //渠道是否有配置
                    for (HashMap hm : dataList) {
                        //配置渠道与主渠道相同，SPID相同，当前月份与已领取月份不相同
                        if ("*".equals(hm.get("CHANNEL"))
                                && spid.equals(hm.get("SPID"))
                                && this.valiConfDate(userid, spid, activityBean, hm)) {
                            contains = true;
                            break;
                        }
                    }
                }
            }

            return contains;

        } catch (Exception e) {
            log.info("校验手动赠送配置失败" + e.getMessage() + userid);
            throw e;
        }

    }


    /**
     * 验证业务是否在配置时间内
     *
     * @param userid
     * @param spid
     * @param activityBean
     * @param confmap
     * @return
     */
    private boolean valiConfDate(String userid, String spid, ActivityBean activityBean, HashMap confmap) {
        boolean res = false;
        String starttime = null;
        String endtime = null;

        String systime = new SimpleDateFormat("yyyy-MM").format(new Date());

        //判断开始时间和结束时间是否合法
        if (confmap.get("STARTTIME") != null && confmap.get("STARTTIME").toString().length() == 14) {
            starttime = confmap.get("STARTTIME").toString();
        }
        if (confmap.get("ENDTIME") != null && confmap.get("ENDTIME").toString().length() == 14) {
            endtime = confmap.get("ENDTIME").toString();
        }

        //有时间配置
        if (!StringUtils.isEmpty(starttime) || !StringUtils.isEmpty(endtime)) {
            //开始时间和结束时间都有配置
            if (!StringUtils.isEmpty(starttime) && !StringUtils.isEmpty(endtime)
                    && Long.parseLong(systime) >= Long.parseLong(starttime)
                    && Long.parseLong(systime) <= Long.parseLong(endtime)) {
                res = true;
                //只配置了开始时间
            } else if (!StringUtils.isEmpty(starttime) && StringUtils.isEmpty(endtime)
                    && Long.parseLong(systime) >= Long.parseLong(starttime)) {
                res = true;
                //只配置了结束时间
            } else if (StringUtils.isEmpty(starttime) && !StringUtils.isEmpty(endtime)
                    && Long.parseLong(systime) <= Long.parseLong(endtime)) {
                res = true;
            }
        } else { //时间无配置
            res = true;
        }

        if (res) {//时间验证通过
            //判断当月是否已领取过会员（系统时间的月份与已领取时间的月份相同）
            if ("0".equals(activityBean.getVipstatus()) && !StringUtils.isEmpty(activityBean.getViptime()) && systime.equals(activityBean.getViptime().substring(0, 7))) {
                log.info("userid=>" + userid + "，spid=>" + spid + "，当月已领取");
                res = false;
            }
        }

        log.info("userid=>" + userid + "，spid=>" + spid + "，手动领取时间验证结果=>" + res + "，systime=>" + systime + "，starttime=>" + starttime + "，endtime=>" + endtime + "，viptime=>" + activityBean.getViptime());

        return res;
    }


    /**
     * 权益领取
     *
     * @param object
     * @return
     * @throws Exception
     */
    public void receive(JSONObject object) throws Exception {
        HashMap<String, String> logmap = new HashMap<>();
        logmap.put("权益领取入参", JSON.toJSONString(object));
        ReceiveEquityLog receiveEquityLog = new ReceiveEquityLog();
        String result = "";
        String resultInfo = "";
        try {
            check(object);
            receiveEquityLog.setReqparam(JSON.toJSONString(object));
            receiveEquityLog.setSerialid(object.getString("serialid"));
            receiveEquityLog.setReceivetype(object.getString("receivetype"));
            String spid = object.getString("productid");
            String userid = object.getString("usermob");
            String spidSon = object.getString("equitycode");  // 单产品的spid 、 复合产品的子产品的spid
            receiveEquityLog.setEquitycode(spidSon);
            receiveEquityLog.setUserid(userid);
            receiveEquityLog.setSpid(spid);
            String activetype = qureyEquityDao.getActivetype(spid);
            receiveEquityLog.setActivetype(activetype);
            Map<String, String> infoMap = qureyEquityDao.getActiveConfig(spidSon, activetype);
            String apitype = infoMap.get("APITYPE");
            String cpid = infoMap.get("CPID");   // 单产品的cpid 、 复合产品的子产品的cpid
            if (!StringUtils.isEmpty(apitype)) {
                switch (apitype) {
                    case "1":
                        // 单产品
                        result = joinVip(userid, spid, activetype, logmap);
                        resultInfo = "joinVip:" + result;
                        break;
                    case "2":
                        // 复合
                        result = joinGroupVip(userid, spid, activetype, logmap);
                        resultInfo = "joinGroupVip:" + result;
                        break;
                    case "3":
                        // 多选一
                        result = joinMulVip(userid, spid, activetype, cpid, logmap);
                        resultInfo = "joinMulVip:" + result;
                        break;
                    default:
                        log.info("TB_ACTIVITY_VIP_CONFIG配置错误" + activetype);
                        throw new BussinessException("9058", "无业务配置信息");
                }
                receiveEquityLog.setJoininfo(result);

                final JSONObject js = JSON.parseObject(result);
                String resultcode = js.getString("resultcode");
                String errorinfo = js.getString("errorinfo");
                if (!"0".equals(resultcode)) {
                    receiveEquityLog.setJoininfo(resultInfo);
                    throw new BussinessException(resultcode, errorinfo);
                }
            } else {
                log.info("缺失业务配置,apitype:" + apitype + ", activetype" + activetype);
                throw new BussinessException("8004", "缺失业务配置");
            }

        } catch (Exception e) {
            e.printStackTrace();
            log.info("权益领取异常" + e.getMessage());
            if (!(e instanceof BussinessException)) {
                if (e.getMessage().length() > 2000) {
                    receiveEquityLog.setJoininfo(e.getMessage().substring(0, 2000));
                } else {
                    receiveEquityLog.setJoininfo(e.getMessage());
                }
            }
            throw e;
        } finally {
            log.info(JSON.toJSONString(logmap));
            // 保存能开领取日志
            try {
                qureyEquityDao.saveReceiveEquityLog(receiveEquityLog);
            } catch (Exception e) {
                e.printStackTrace();
                log.info("保存能开领取记录失败" + e.getMessage());
            }
        }
    }

    private String joinGroupVip(String userid, String spid, String activetype, HashMap<String, String> logMap) throws Exception {
//        String url = "http://testapi.mige.tv/video-activity/joinGroupVip";
        ProJinVokeUrlBean proJinVokeUrlBean = proJinVokeUrlDao.getProConf("videoactivity", "joinGroupVip");
        String url = proJinVokeUrlBean.getInvokeurl();
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String pwd = qureyEquityDao.getPwd(spid);
        userid = DesUtil.encode(userid, pwd);
        String signature = MD5.MD5Encode(activetype + userid + timestamp + pwd);
        userid = URLEncoder.encode(userid, "utf-8");
        url = addParam(url, "userid", userid);
        url = addParam(url, "timestamp", timestamp);
        url = addParam(url, "activetype", activetype);
        url = addParam(url, "signature", signature);
        logMap.put("joinGroupVip请求报文", url);
        String result = URLUtil.get(url);
        logMap.put("响应报文", result);
        return result;
    }

    private String joinMulVip(String userid, String spid, String activetype, String cpid, HashMap<String, String> logMap) throws Exception {
//        String url = "http://testapi.mige.tv/video-activity/joinMultiVip";
        ProJinVokeUrlBean proJinVokeUrlBean = proJinVokeUrlDao.getProConf("videoactivity", "joinMultiVip");
        String url = proJinVokeUrlBean.getInvokeurl();
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String pwd = qureyEquityDao.getPwd(spid);
        userid = DesUtil.encode(userid, pwd);
        String signature = MD5.MD5Encode(activetype + userid + cpid + timestamp + pwd);
        userid = URLEncoder.encode(userid, "utf-8");

        url = addParam(url, "userid", userid);
        url = addParam(url, "timestamp", timestamp);
        url = addParam(url, "activetype", activetype);
        url = addParam(url, "signature", signature);
        url = addParam(url, "givecpid", cpid);
        logMap.put("joinMulVip请求报文", url);
        String result = URLUtil.get(url);
        logMap.put("响应报文", result);
        return result;
    }

    private String joinVip(String userid, String spid, String activetype, HashMap<String, String> logMap) throws Exception {
//        String url = "http://testapi.mige.tv/video-activity/joinActivity";
        ProJinVokeUrlBean proJinVokeUrlBean = proJinVokeUrlDao.getProConf("videoactivity", "joinActivity");
        String url = proJinVokeUrlBean.getInvokeurl();
        String userval = userid;
        String timestamp = System.currentTimeMillis() / 1000 + "";
        String pwd = qureyEquityDao.getPwd(spid);
        userid = DesUtil.encode(userid, pwd);
        String signature = MD5.MD5Encode(userid + userval + pwd + timestamp);
        String enUserid = URLEncoder.encode(userid, "utf-8");
        url = addParam(url, "userid", enUserid);
        url = addParam(url, "timestamp", timestamp);
        url = addParam(url, "signature", signature);
        url = addParam(url, "activetype", activetype);
        url = addParam(url, "usertype", "4");
        url = addParam(url, "userval", userval);
        logMap.put("joinVip请求报文", url);
        String result = URLUtil.get(url);
        logMap.put("响应报文", result);
        return result;
    }


    private void check(JSONObject object) throws Exception {
        if (StringUtils.isEmpty(object.getString("usermob")) || StringUtils.isEmpty(object.getString("productid")) || StringUtils.isEmpty(object.getString("equitycode"))) {
            throw new BussinessException("80003", "参数缺失");
        }
    }


    public static String addParam(String url, String key, String value) {
        if (value == null)
            return url;
        if (!url.contains("?")) {
            url += "?";
        } else {
            url += "&";
        }
        return url + key + "=" + value;
    }
}
